@@ -31,7 +31,7 @@ xmlXPathObject* eval_common(const Glib::ustring& xpath,
   const xmlpp::Node::PrefixNsMap* namespaces,
   xmlpp::XPathResultType* result_type, xmlNode* node)
 {
-  xmlXPathContext* ctxt = xmlXPathNewContext(node->doc);
+  auto ctxt = xmlXPathNewContext(node->doc);
   if (!ctxt)
     throw xmlpp::internal_error("Could not create XPath context for " + xpath);
   ctxt->node = node;
@@ -45,7 +45,7 @@ xmlXPathObject* eval_common(const Glib::ustring& xpath,
         reinterpret_cast<const xmlChar*>(it->second.c_str()));
   }
 
-  xmlXPathObject* xpath_value = xmlXPathEvalExpression(
+  auto xpath_value = xmlXPathEvalExpression(
     reinterpret_cast<const xmlChar*>(xpath.c_str()), ctxt);
 
   xmlXPathFreeContext(ctxt);
@@ -77,7 +77,7 @@ bool eval_common_to_boolean(const Glib::ustring& xpath,
   const xmlpp::Node::PrefixNsMap* namespaces,
   xmlpp::XPathResultType* result_type, xmlNode* node)
 {
-  xmlXPathObject* xpath_value = eval_common(xpath, namespaces, result_type, node);
+  auto xpath_value = eval_common(xpath, namespaces, result_type, node);
   const int result = xmlXPathCastToBoolean(xpath_value);
   xmlXPathFreeObject(xpath_value);
   return static_cast<bool>(result);
@@ -88,7 +88,7 @@ double eval_common_to_number(const Glib::ustring& xpath,
   const xmlpp::Node::PrefixNsMap* namespaces,
   xmlpp::XPathResultType* result_type, xmlNode* node)
 {
-  xmlXPathObject* xpath_value = eval_common(xpath, namespaces, result_type, node);
+  auto xpath_value = eval_common(xpath, namespaces, result_type, node);
   const double result = xmlXPathCastToNumber(xpath_value);
   xmlXPathFreeObject(xpath_value);
   return result;
@@ -99,7 +99,7 @@ Glib::ustring eval_common_to_string(const Glib::ustring& xpath,
   const xmlpp::Node::PrefixNsMap* namespaces,
   xmlpp::XPathResultType* result_type, xmlNode* node)
 {
-  xmlXPathObject* xpath_value = eval_common(xpath, namespaces, result_type, node);
+  auto xpath_value = eval_common(xpath, namespaces, result_type, node);
   xmlChar* result = xmlXPathCastToString(xpath_value);
   xmlXPathFreeObject(xpath_value);
   if (result)
@@ -111,6 +111,18 @@ Glib::ustring eval_common_to_string(const Glib::ustring& xpath,
   return Glib::ustring();
 }
 
+// Common part of all add_child*() methods.
+xmlpp::Element* add_child_common(const Glib::ustring& name, xmlNode* child, xmlNode* node)
+{
+  if (!node)
+  {
+    xmlFreeNode(child);
+    throw xmlpp::internal_error("Could not add child element node " + name);
+  }
+  xmlpp::Node::create_wrapper(node);
+  return static_cast<xmlpp::Element*>(node->_private);
+}
+
 } // anonymous namespace
 
 namespace xmlpp
@@ -172,7 +184,7 @@ Node* Node::get_previous_sibling()
 
 static Node* _convert_node(xmlNode* node)
 {
-  Node* res = 0;
+  Node* res = nullptr;
   if(node)
   {
     Node::create_wrapper(node);
@@ -183,7 +195,7 @@ static Node* _convert_node(xmlNode* node)
 
 Node* Node::get_first_child(const Glib::ustring& name)
 {
-  xmlNode* child = impl_->children;
+  auto child = impl_->children;
   if(!child)
     return 0;
 
@@ -204,7 +216,7 @@ const Node* Node::get_first_child(const Glib::ustring& name) const
 
 Node::NodeList Node::get_children(const Glib::ustring& name)
 {
-   xmlNode* child = impl_->children;
+   auto child = impl_->children;
    if(!child)
      return NodeList();
 
@@ -227,56 +239,70 @@ const Node::NodeList Node::get_children(const Glib::ustring& name) const
 Element* Node::add_child(const Glib::ustring& name,
                          const Glib::ustring& ns_prefix)
 {
-  _xmlNode* child = create_new_child_node(name, ns_prefix);
-  _xmlNode* node = xmlAddChild(impl_, child);
-  if (!node)
-  {
-    xmlFreeNode(child);
-    throw internal_error("Could not add child element node " + name);
-  }
-  Node::create_wrapper(node);
-  return static_cast<Element*>(node->_private);
+  auto child = create_new_child_node(name, ns_prefix);
+  auto node = xmlAddChild(impl_, child);
+  return add_child_common(name, child, node);
 }
 
 Element* Node::add_child(xmlpp::Node* previous_sibling, 
                          const Glib::ustring& name,
                          const Glib::ustring& ns_prefix)
 {
-  if(!previous_sibling)
+  if (!previous_sibling)
     return 0;
 
-  _xmlNode* child = create_new_child_node(name, ns_prefix);
-  _xmlNode* node = xmlAddNextSibling(previous_sibling->cobj(), child);
-  if (!node)
-  {
-    xmlFreeNode(child);
-    throw internal_error("Could not add child element node " + name);
-  }
-  Node::create_wrapper(node);
-  return static_cast<Element*>(node->_private);
+  auto child = create_new_child_node(name, ns_prefix);
+  auto node = xmlAddNextSibling(previous_sibling->cobj(), child);
+  return add_child_common(name, child, node);
 }
 
 Element* Node::add_child_before(xmlpp::Node* next_sibling, 
                          const Glib::ustring& name,
                          const Glib::ustring& ns_prefix)
 {
-  if(!next_sibling)
+  if (!next_sibling)
     return 0;
 
-  _xmlNode* child = create_new_child_node(name, ns_prefix);
-  _xmlNode* node = xmlAddPrevSibling(next_sibling->cobj(), child);
-  if (!node)
-  {
-    xmlFreeNode(child);
-    throw internal_error("Could not add child element node " + name);
-  }
-  Node::create_wrapper(node);
-  return static_cast<Element*>(node->_private);
+  auto child = create_new_child_node(name, ns_prefix);
+  auto node = xmlAddPrevSibling(next_sibling->cobj(), child);
+  return add_child_common(name, child, node);
+}
+
+Element* Node::add_child_with_new_ns(const Glib::ustring& name,
+  const Glib::ustring& ns_uri, const Glib::ustring& ns_prefix)
+{
+  auto child = create_new_child_node_with_new_ns(name, ns_uri, ns_prefix);
+  auto node = xmlAddChild(impl_, child);
+  return add_child_common(name, child, node);
+}
+
+Element* Node::add_child_with_new_ns(xmlpp::Node* previous_sibling,
+  const Glib::ustring& name,
+  const Glib::ustring& ns_uri, const Glib::ustring& ns_prefix)
+{
+  if (!previous_sibling)
+    return 0;
+
+  auto child = create_new_child_node_with_new_ns(name, ns_uri, ns_prefix);
+  auto node = xmlAddNextSibling(previous_sibling->cobj(), child);
+  return add_child_common(name, child, node);
+}
+
+Element* Node::add_child_before_with_new_ns(xmlpp::Node* next_sibling,
+  const Glib::ustring& name,
+  const Glib::ustring& ns_uri, const Glib::ustring& ns_prefix)
+{
+  if (!next_sibling)
+    return 0;
+
+  auto child = create_new_child_node_with_new_ns(name, ns_uri, ns_prefix);
+  auto node = xmlAddPrevSibling(next_sibling->cobj(), child);
+  return add_child_common(name, child, node);
 }
 
 _xmlNode* Node::create_new_child_node(const Glib::ustring& name, const Glib::ustring& ns_prefix)
 {
-   xmlNs* ns = 0;
+   xmlNs* ns = nullptr;
 
    if(impl_->type != XML_ELEMENT_NODE)
    {
@@ -301,6 +327,36 @@ _xmlNode* Node::create_new_child_node(const Glib::ustring& name, const Glib::ust
    return xmlNewNode(ns, (const xmlChar*)name.c_str());
 }
 
+_xmlNode* Node::create_new_child_node_with_new_ns(const Glib::ustring& name,
+  const Glib::ustring& ns_uri, const Glib::ustring& ns_prefix)
+{
+  if (impl_->type != XML_ELEMENT_NODE)
+    throw internal_error("You can only add child nodes to element nodes.");
+
+  auto child = xmlNewNode(0, (const xmlChar*)name.c_str());
+  if (!child)
+    throw internal_error("Could not create new element node.");
+
+  auto ns = xmlNewNs(child, (const xmlChar*)(ns_uri.empty() ? 0 : ns_uri.c_str()),
+                       (const xmlChar*)(ns_prefix.empty() ? 0 : ns_prefix.c_str()) );
+  // xmlNewNs() does not create a namespace node for the predefined xml prefix.
+  // It's usually defined in the document and not in any specific node.
+  if (!ns && ns_prefix == "xml")
+  {
+    ns = xmlSearchNs(impl_->doc, impl_, (const xmlChar*)ns_prefix.c_str());
+    if (ns && (ns_uri != (ns->href ? (const char*)ns->href : "")))
+      ns = nullptr;
+  }
+  if (!ns)
+  {
+    xmlFreeNode(child);
+    throw internal_error("Could not create new namespace node.");
+  }
+
+  xmlSetNs(child, ns);
+
+  return child;
+}
 
 void Node::remove_child(Node* node)
 {
@@ -308,7 +364,7 @@ void Node::remove_child(Node* node)
   //This would require a more complex memory management API.
   if (!node)
     return;
-  xmlNode* cnode = node->cobj();
+  auto cnode = node->cobj();
   Node::free_wrappers(cnode); //This delete the C++ node (not this) itself.
   xmlUnlinkNode(cnode);
   xmlFreeNode(cnode);
@@ -320,7 +376,7 @@ Node* Node::import_node(const Node* node, bool recursive)
     return 0;
 
   //Create the node, by copying:
-  xmlNode* imported_node = xmlDocCopyNode(const_cast<xmlNode*>(node->cobj()), impl_->doc, recursive);
+  auto imported_node = xmlDocCopyNode(const_cast<xmlNode*>(node->cobj()), impl_->doc, recursive);
   if (!imported_node)
   {
     throw exception("Unable to copy the node that shall be imported");
@@ -328,7 +384,7 @@ Node* Node::import_node(const Node* node, bool recursive)
 
   if (imported_node->type == XML_ATTRIBUTE_NODE && impl_->type == XML_ELEMENT_NODE)
   {
-    xmlAttr* old_attr = xmlHasNsProp(impl_, imported_node->name,
+    auto old_attr = xmlHasNsProp(impl_, imported_node->name,
       imported_node->ns ? imported_node->ns->href : 0);
     if (old_attr && old_attr->type != XML_ATTRIBUTE_DECL)
     {
@@ -340,7 +396,7 @@ Node* Node::import_node(const Node* node, bool recursive)
   }
 
   //Add the node:
-  xmlNode* added_node = xmlAddChild(this->cobj(), imported_node);
+  auto added_node = xmlAddChild(this->cobj(), imported_node);
 
   if (!added_node)
   {
@@ -393,7 +449,7 @@ Glib::ustring Node::get_path() const
 
 static NodeSet find_impl(xmlXPathContext* ctxt, const Glib::ustring& xpath)
 {
-  xmlXPathObject* result = xmlXPathEval((const xmlChar*)xpath.c_str(), ctxt);
+  auto result = xmlXPathEval((const xmlChar*)xpath.c_str(), ctxt);
 
   if(!result)
   {
@@ -410,7 +466,7 @@ static NodeSet find_impl(xmlXPathContext* ctxt, const Glib::ustring& xpath)
     throw internal_error("Only nodeset result types are supported.");
   }
 
-  xmlNodeSet* nodeset = result->nodesetval;
+  auto nodeset = result->nodesetval;
   NodeSet nodes;
   if( nodeset && !xmlXPathNodeSetIsEmpty(nodeset))
   {
@@ -418,7 +474,7 @@ static NodeSet find_impl(xmlXPathContext* ctxt, const Glib::ustring& xpath)
     nodes.reserve(count);
     for (int i = 0; i != count; ++i)
     {
-      xmlNode* cnode = xmlXPathNodeSetItem(nodeset, i);
+      auto cnode = xmlXPathNodeSetItem(nodeset, i);
       if(!cnode)
       {
         std::cerr << "Node::find_impl: The xmlNode was null." << std::endl;
@@ -436,7 +492,7 @@ static NodeSet find_impl(xmlXPathContext* ctxt, const Glib::ustring& xpath)
       //TODO: Check for other cnode->type values?
   
       Node::create_wrapper(cnode);
-      Node* cppNode = static_cast<Node*>(cnode->_private);
+      auto cppNode = static_cast<Node*>(cnode->_private);
       nodes.push_back(cppNode);
     }
   }
@@ -453,7 +509,7 @@ static NodeSet find_impl(xmlXPathContext* ctxt, const Glib::ustring& xpath)
 
 NodeSet Node::find(const Glib::ustring& xpath) const
 {
-  xmlXPathContext* ctxt = xmlXPathNewContext(impl_->doc);
+  auto ctxt = xmlXPathNewContext(impl_->doc);
   if (!ctxt)
     throw internal_error("Could not create XPath context for " + xpath);
   ctxt->node = impl_;
@@ -464,7 +520,7 @@ NodeSet Node::find(const Glib::ustring& xpath) const
 NodeSet Node::find(const Glib::ustring& xpath,
 		   const PrefixNsMap& namespaces) const
 {
-  xmlXPathContext* ctxt = xmlXPathNewContext(impl_->doc);
+  auto ctxt = xmlXPathNewContext(impl_->doc);
   if (!ctxt)
     throw internal_error("Could not create XPath context for " + xpath);
   ctxt->node = impl_;
@@ -566,7 +622,7 @@ void Node::set_namespace(const Glib::ustring& ns_prefix)
   }
 
   //Look for the existing namespace to use:
-  xmlNs* ns = xmlSearchNs( cobj()->doc, cobj(), (xmlChar*)(ns_prefix.empty() ? 0 : ns_prefix.c_str()) );
+  auto ns = xmlSearchNs( cobj()->doc, cobj(), (xmlChar*)(ns_prefix.empty() ? 0 : ns_prefix.c_str()) );
   if(ns)
   {
       //Use it for this element:
@@ -683,7 +739,7 @@ void Node::free_wrappers(xmlNode* node)
   if (node->type != XML_ENTITY_REF_NODE)
   {
     //Walk the children list.
-    for (xmlNode* child = node->children; child; child = child->next)
+    for (auto child = node->children; child; child = child->next)
       free_wrappers(child);
   }
 
@@ -693,21 +749,21 @@ void Node::free_wrappers(xmlNode* node)
     //Node types that have no properties
     case XML_DTD_NODE:
       delete static_cast<Dtd*>(node->_private);
-      node->_private = 0;
+      node->_private = nullptr;
       return;
     case XML_ATTRIBUTE_NODE:
     case XML_ELEMENT_DECL:
     case XML_ATTRIBUTE_DECL:
     case XML_ENTITY_DECL:
       delete static_cast<Node*>(node->_private);
-      node->_private = 0;
+      node->_private = nullptr;
       return;
     case XML_DOCUMENT_NODE:
       //Do not free now. The Document is usually the one who owns the caller.
       return;
     default:
       delete static_cast<Node*>(node->_private);
-      node->_private = 0;
+      node->_private = nullptr;
       break;
   }
 
@@ -716,7 +772,7 @@ void Node::free_wrappers(xmlNode* node)
   //_xmlNode::properties would be a nonsense value, leading to crashes,
   //(and shown as valgrind warnings), so we return above, to avoid 
   //checking it here.
-  for(xmlAttr* attr = node->properties; attr; attr = attr->next)
+  for(auto attr = node->properties; attr; attr = attr->next)
     free_wrappers(reinterpret_cast<xmlNode*>(attr));
 }
 
