@@ -15,23 +15,19 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <libxml++/libxml++.h>
-
 #include <cstdlib>
 #include <glibmm.h>
 #include <sstream>
 #include <stdexcept>
 
+#include <libxml++/libxml++.h>
+
 class MySaxParser : public xmlpp::SaxParser
 {
-public:
-  bool throw_on_start_doc = true;
-
 protected:
   void on_start_document() override
   {
-    if (throw_on_start_doc)
-      throw std::runtime_error("some custom runtime exception");
+    throw std::runtime_error("some custom runtime exception");
   }
   void on_error(const Glib::ustring& /* text */) override
   {
@@ -49,17 +45,10 @@ int main()
     bool exceptionThrown = false;
     try
     {
-      // Depending on the libxml2 version, MySaxParser::on_start_document()
-      // may or may not be called before MySaxParser::on_error().
-      parser.throw_on_start_doc = false;
       parser.parse_chunk("<?");
       parser.finish_chunk_parsing();
     }
-#ifdef LIBXMLXX_HAVE_EXCEPTION_PTR
     catch(const std::runtime_error& e)
-#else
-    catch(const xmlpp::exception& e)
-#endif
     {
       exceptionThrown = true;
       g_assert_cmpstr(e.what(), ==, "on_error() called");
@@ -70,7 +59,6 @@ int main()
     exceptionThrown = false;
     try
     {
-      parser.throw_on_start_doc = true;
       std::stringstream ss("<root></root>");
       parser.parse_stream(ss);
     }
@@ -80,11 +68,7 @@ int main()
       // error should not have been thrown.
       g_assert_not_reached();
     }
-#ifdef LIBXMLXX_HAVE_EXCEPTION_PTR
     catch(const std::runtime_error& e)
-#else
-    catch(const xmlpp::exception& e)
-#endif
     {
       exceptionThrown = true;
       g_assert_cmpstr(e.what(), ==, "some custom runtime exception");
